import csv
import os
import re
import sqlite3

from PyQt5.QtCore import QThread
from PyQt5.QtWidgets import QFileDialog, QProgressBar
from qgis._core import QgsVectorFileWriter, QgsCoordinateTransformContext
from shapely import wkt

from qgis.PyQt import QtWidgets, uic, QtCore
from qgis.PyQt.QtCore import pyqtSignal

FORM_CLASS, _ = uic.loadUiType(os.path.join(
    os.path.dirname(__file__), 'cyanlove_export_tab_kml_base.ui'))


class cyanlove_export_tab_kml(QtWidgets.QDockWidget, FORM_CLASS):
    closingPlugin = pyqtSignal()

    def __init__(self, parent=None):
        super(cyanlove_export_tab_kml, self).__init__(parent)
        self.export_thread = None
        self.setupUi(self)
        self.setFloating(True)
        self.setWindowFlags(QtCore.Qt.Dialog)  # 使对话框始终在前
        self.setWindowModality(QtCore.Qt.ApplicationModal)  # 或者使用 Qt.WindowModal
        self.btn_export_tab.clicked.connect(self.export_tab_kml)
        self.btn_export_kml.clicked.connect(self.export_tab_kml)

    def update_label(self, value):
        self.label_3.setText(value)  # 更新值

    def export_finished(self):
        print("导出完成！")
        self.label_3.setText('导出完毕!')

    def export_tab_kml(self):
        button = self.sender()
        button_name = ""
        if button:
            # 获取按钮的文本
            button_name = button.objectName()
            print(button_name)
        options = QFileDialog.Options()  # 创建文件对话框选项
        options |= QFileDialog.ReadOnly  # 可以设置一些选项，例如只读
        filterfile = "MapInfo TAB Files (*.tab)"
        if button_name == "btn_export_kml":
            filterfile = "KML File (*.kml)"
        file_name, _ = QFileDialog.getSaveFileName(self, "选择文件", "", filterfile, options=options)

        # 检查用户是否选择了文件
        if file_name:
            selected_layer = self.mMapLayerComboBox.currentLayer()
            if selected_layer is None:
                print("未选择有效图层!")
            else:
                print(f"选择的图层: {selected_layer.name()}")
                csv_file_path = file_name
                print(f"保存的文件：{csv_file_path}")

                # 启动导出线程
                self.export_thread = ExportThread(selected_layer, csv_file_path, button_name)
                self.export_thread.updatelabel.connect(self.update_label)
                self.export_thread.finished.connect(self.export_finished)  # 导出完成后连接到槽
                self.export_thread.start()  # 启动线程


        else:
            print("用户取消了文件选择")  # 处理取消选择的情况


class ExportThread(QThread):
    updatelabel = pyqtSignal(str)  # 声明一个更新label新增信号

    def __init__(self, selected_layer, savpath, button_name):
        super().__init__()  # 初始化父类
        self.savpath = savpath  # 存储
        self.button_name = button_name  # 按钮名称
        self.selected_layer = selected_layer  # QGIS 图层

    def run(self):
        self.updatelabel.emit('正在导出中...')
        field_names = [f"{field.name()}" for field in self.selected_layer.fields()]
        # 导出TAB格式
        if self.button_name == "btn_export_tab":
            # 设置导出参数
            options = QgsVectorFileWriter.SaveVectorOptions()

            options.driverName = 'MapInfo File'  # 指定输出格式为MapInfo TAB
            options.fileEncoding = 'cp936'  # 设置文件编码为cp936
            transform_context = QgsCoordinateTransformContext()  # 创建坐标转换上下文
            # 执行导出
            error = QgsVectorFileWriter.writeAsVectorFormatV3(self.selected_layer, self.savpath, transform_context,
                                                              options)

            if error[0] == QgsVectorFileWriter.NoError:
                print("导出成功！")
                # 检查是否能成功调用写入函数
                try:
                    self.writertabfilecolumn_to_gbk(self.savpath, field_names)
                    self.updatelabel.emit('导出成功!')
                except Exception as e:
                    print("写入表格时出现错误:", e)
                    self.updatelabel.emit(f'写入表格失败: {e}')
            else:
                print("导出失败：", error[1])
                self.updatelabel.emit(f'导出失败{error[1]}')

        if self.button_name == "btn_export_kml":
            # 设置导出参数
            options = QgsVectorFileWriter.SaveVectorOptions()
            options.driverName = 'KML'  # 指定输出格式为MapInfo TAB
            options.fileEncoding = 'utf-8'  # 设置文件编码为cp936
            transform_context = QgsCoordinateTransformContext()  # 创建坐标转换上下文
            # 执行导出
            error = QgsVectorFileWriter.writeAsVectorFormatV3(self.selected_layer, self.savpath, transform_context,
                                                              options)

            if error[0] == QgsVectorFileWriter.NoError:
                print("导出成功！")
                self.updatelabel.emit('导出成功!')
            else:
                print("导出失败：", error[1])
                self.updatelabel.emit(f'导出失败{error[1]}')

    def writertabfilecolumn_to_gbk(self, file_path, field_names):
        print("修改表头")
        # 读取原文件内容
        fields_list = []
        try:
            with open(file_path, 'r', encoding='utf-8', errors='replace') as file:
                content = file.readlines()
        except Exception as e:
            print(f"发生错误: {e}")
            self.updatelabel.emit(f'导出失败{e}')
            content = []

        if content:
            for i, line in enumerate(content):
                if line.strip().startswith("Fields"):
                    # 提取字段类型，将其写入数组
                    fields_list.extend(self.rx_to_str(line) for line in content[i + 1:] if line.strip())
                    break  # 找到后可结束循环

        # 字段和字段类型合并

        result = list(map(lambda x: f"\"{x[0]}\" {x[1]}", zip(field_names, fields_list)))
        # print(result)
        # 合并格式，替换
        header = ("!table\n"
                  "!version 450\n"
                  "!charset WindowsSimpChinese\n\n"
                  "Definition Table\n"
                  "  Type NATIVE Charset \"WindowsSimpChinese\"\n"
                  "  Fields {}\n").format(len(field_names))

        fields_string = ""
        for field in result:
            fields_string += "    {}\n".format(field)
        table_definition = header + fields_string
        print(table_definition)
        try:

            with open(file_path, 'w', encoding='gbk', errors='replace') as file:
                file.write(table_definition)  # 写入字符串
        except Exception as e:
            print(f"发生错误: {e}")
            self.updatelabel.emit(f'导出失败{e}')

    @staticmethod
    def rx_to_str(textstr):
        data_types = [
            "Char",
            "Integer",
            "Smallint",
            "Float",
            "Decimal",
            "Date",
            "Time",
            "DateTime",
            "Logical"
        ]
        # 将数据类型转换为小写并创建正则表达式模式
        lower_data_types = [dtype.lower() for dtype in data_types]
        pattern = r'(' + '|'.join(re.escape(dtype) for dtype in lower_data_types) + r')\s*(\([^)]*\))?\s*;'

        # 将输入字符串转为小写
        lower_textstr = textstr.lower()

        # 从右向左查找匹配
        matches = re.findall(pattern, lower_textstr)

        # 如果找到匹配，则返回匹配项
        if matches:
            last_match = matches[-1]  # 获取最后一个匹配
            return f"{last_match[0]} {last_match[1] if last_match[1] else ''} ;".strip()  # 返回数据类型和可能的长度
        return None
