import math


class CoordinateConverter:
    x_pi = 3.14159265358979324 * 3000.0 / 180.0
    a = 6378245.0  # WGS84椭球体长半轴
    ee = 0.00669342162296594323  # WGS84椭球体偏心率平方

    @staticmethod
    def gcj02_to_bd09(lng, lat):
        z = math.sqrt(lng * lng + lat * lat) + 0.00002 * math.sin(lat * CoordinateConverter.x_pi)
        theta = math.atan2(lat, lng) + 0.000003 * math.cos(lng * CoordinateConverter.x_pi)
        bd_lng = z * math.cos(theta) + 0.0065
        bd_lat = z * math.sin(theta) + 0.006
        return [bd_lng, bd_lat]

    @staticmethod
    def bd09_to_gcj02(bd_lon, bd_lat):
        x = bd_lon - 0.0065
        y = bd_lat - 0.006
        z = math.sqrt(x * x + y * y) - 0.00002 * math.sin(y * CoordinateConverter.x_pi)
        theta = math.atan2(y, x) - 0.000003 * math.cos(x * CoordinateConverter.x_pi)
        gg_lng = z * math.cos(theta)
        gg_lat = z * math.sin(theta)
        return [gg_lng, gg_lat]

    @staticmethod
    def wgs84_to_gcj02(lng, lat):
        if CoordinateConverter.out_of_china(lng, lat):
            return lng, lat
        dlat = CoordinateConverter._transformlat(lng - 105.0, lat - 35.0)
        dlng = CoordinateConverter._transformlng(lng - 105.0, lat - 35.0)
        radlat = lat / 180.0 * math.pi
        magic = math.sin(radlat)
        magic = 1 - CoordinateConverter.ee * magic * magic
        sqrtmagic = math.sqrt(magic)
        dlat = (dlat * 180.0) / ((CoordinateConverter.a * (1 - CoordinateConverter.ee)) / (magic * sqrtmagic) * math.pi)
        dlng = (dlng * 180.0) / (CoordinateConverter.a / sqrtmagic * math.cos(radlat) * math.pi)
        mglat = lat + dlat
        mglng = lng + dlng
        return [mglng, mglat]

    @staticmethod
    def gcj02_to_wgs84(lng, lat):
        if CoordinateConverter.out_of_china(lng, lat):
            return lng, lat
        dlat = CoordinateConverter._transformlat(lng - 105.0, lat - 35.0)
        dlng = CoordinateConverter._transformlng(lng - 105.0, lat - 35.0)
        radlat = lat / 180.0 * math.pi
        magic = math.sin(radlat)
        magic = 1 - CoordinateConverter.ee * magic * magic
        sqrtmagic = math.sqrt(magic)
        dlat = (dlat * 180.0) / ((CoordinateConverter.a * (1 - CoordinateConverter.ee)) / (magic * sqrtmagic) * math.pi)
        dlng = (dlng * 180.0) / (CoordinateConverter.a / sqrtmagic * math.cos(radlat) * math.pi)
        mglat = lat + dlat
        mglng = lng + dlng
        return [lng * 2 - mglng, lat * 2 - mglat]

    @staticmethod
    def bd09_to_wgs84(bd_lon, bd_lat):
        lon, lat = CoordinateConverter.bd09_to_gcj02(bd_lon, bd_lat)
        return CoordinateConverter.gcj02_to_wgs84(lon, lat)

    @staticmethod
    def wgs84_to_bd09(lon, lat):
        lon, lat = CoordinateConverter.wgs84_to_gcj02(lon, lat)
        return CoordinateConverter.gcj02_to_bd09(lon, lat)

    @staticmethod
    def _transformlat(lng, lat):
        ret = -100.0 + 2.0 * lng + 3.0 * lat + 0.2 * lat * lat + \
              0.1 * lng * lat + 0.2 * math.sqrt(math.fabs(lng))
        ret += (20.0 * math.sin(6.0 * lng * math.pi) + 20.0 *
                math.sin(2.0 * lng * math.pi)) * 2.0 / 3.0
        ret += (20.0 * math.sin(lat * math.pi) + 40.0 *
                math.sin(lat / 3.0 * math.pi)) * 2.0 / 3.0
        ret += (160.0 * math.sin(lat / 12.0 * math.pi) + 320 *
                math.sin(lat * math.pi / 30.0)) * 2.0 / 3.0
        return ret

    @staticmethod
    def _transformlng(lng, lat):
        ret = 300.0 + lng + 2.0 * lat + 0.1 * lng * lng + \
              0.1 * lng * lat + 0.1 * math.sqrt(math.fabs(lng))
        ret += (20.0 * math.sin(6.0 * lng * math.pi) + 20.0 *
                math.sin(2.0 * lng * math.pi)) * 2.0 / 3.0
        ret += (20.0 * math.sin(lng * math.pi) + 40.0 *
                math.sin(lng / 3.0 * math.pi)) * 2.0 / 3.0
        ret += (150.0 * math.sin(lng / 12.0 * math.pi) + 300.0 *
                math.sin(lng / 30.0 * math.pi)) * 2.0 / 3.0
        return ret

    @staticmethod
    def out_of_china(lng, lat):
        return not (73.66 < lng < 135.05 and 3.86 < lat < 53.55)

    @staticmethod
    def baidu_to_google(lng, lat):
        return CoordinateConverter.bd09_to_wgs84(float(lng), float(lat))

    @staticmethod
    def google_to_baidu(lng, lat):
        return CoordinateConverter.wgs84_to_bd09(float(lng), float(lat))
