from qgis.core import QgsGeometry, QgsVectorLayer, QgsFeature, QgsPointXY, QgsProject

# 定义坐标字符串
coordinates_str = "108.8604076718378_25.835547714994025;108.8604076718378_25.835719414995516;108.86040767155_25.835719414995516;108.86040767155_25.835719414999346;108.86006437159483_25.835719414999346;108.86006437159483_25.835891115000845;108.86006437130703_25.835891115000845;108.86006437130703_25.836062715002335;108.86006437101939_25.836062715002335;108.86006437101939_25.836234415003837;108.86006437073159_25.836234415003837;108.86006437073159_25.83640611500534;108.86006437044378_25.83640611500534;108.86006437044378_25.836406115009172;108.85972107048862_25.836406115009172;108.85972107048862_25.836577715010666;108.85972107020099_25.836577715010666;108.85972107020099_25.83674941501216;108.85972106991316_25.83674941501216;108.85972106991316_25.836921015013658;108.85972106962551_25.836921015013658;108.85972106962551_25.837092715015157"

# 分割坐标字符串并创建点列表
points = []
for coord in coordinates_str.split(";"):
    lon, lat = map(float, coord.split("_"))
    points.append(QgsPointXY(lon, lat))  # 使用 QgsPointXY 创建点

# 创建 MultiPoint 几何
multi_point_geometry = QgsGeometry.fromMultiPointXY(points)

# 创建一个新的矢量图层
layer = QgsVectorLayer("MultiPoint?crs=EPSG:4326", "MultiPoint Layer", "memory")  # 修改为 MultiPoint
provider = layer.dataProvider()

# 创建一个特征并设置几何
feature = QgsFeature()
feature.setGeometry(multi_point_geometry)

# 将特征添加到图层中
provider.addFeatures([feature])
layer.updateExtents()

# 将图层添加到 QGIS 项目中
QgsProject.instance().addMapLayer(layer)

print("MultiPoint 图层已创建并添加到地图中！")


# ========================

from qgis.core import QgsProject, QgsVectorLayer, QgsFeature, QgsGeometry, QgsPointXY, QgsField
from qgis.PyQt.QtCore import QVariant

# 创建一个新的矢量图层（类型为 MultiPolygon）
layer = QgsVectorLayer("MultiPolygon?crs=EPSG:4326", "CenteredHoleMultiPolygon", "memory")

# 检查图层是否有效
if not layer.isValid():
    print("图层无效")
else:
    print("图层创建成功")

# 添加字段（属性）
layer.dataProvider().addAttributes([QgsField("id", QVariant.Int)])
layer.updateFields()

# 定义外部轮廓坐标
outer_points = [
    QgsPointXY(107.9, 24.9),  # 左下角
    QgsPointXY(108.2, 24.9),  # 右下角
    QgsPointXY(108.2, 25.1),  # 右上角
    QgsPointXY(107.9, 25.1),  # 左上角
    QgsPointXY(107.9, 24.9)   # 返回左下角以闭合多边形
]

# 计算外部轮廓的中心点
x_coords = [point.x() for point in outer_points[:-1]]  # 不包括最后一个闭合点
y_coords = [point.y() for point in outer_points[:-1]]
center_x = sum(x_coords) / len(x_coords)
center_y = sum(y_coords) / len(y_coords)
center_point = QgsPointXY(center_x, center_y)

# 定义中心孔洞的坐标，以中心点为基准
hole_size = 0.05  # 孔洞的大小（边长）
hole_points = [
    QgsPointXY(center_x - hole_size / 2, center_y - hole_size / 2),  # 左下角
    QgsPointXY(center_x + hole_size / 2, center_y - hole_size / 2),  # 右下角
    QgsPointXY(center_x + hole_size / 2, center_y + hole_size / 2),  # 右上角
    QgsPointXY(center_x - hole_size / 2, center_y + hole_size / 2),  # 左上角
    QgsPointXY(center_x - hole_size / 2, center_y - hole_size / 2)    # 返回左下角以闭合孔洞
]

# 创建 MultiPolygon
multi_polygon = QgsGeometry.fromMultiPolygonXY([[outer_points, hole_points]])

# 创建要素并添加几何
feature = QgsFeature()
feature.setGeometry(multi_polygon)
feature.setAttributes([1])  # 设置属性

# 将要素添加到图层
layer.dataProvider().addFeature(feature)

# 更新图层
layer.updateExtents()

# 将图层添加到 QGIS 项目
QgsProject.instance().addMapLayer(layer)

print("要素已成功添加到图层，孔洞已移动到外层轮廓中心！")

# =====

from qgis.core import QgsPointXY, QgsGeometry, QgsFeature, QgsVectorLayer, QgsFields, QgsField
from qgis.PyQt.QtCore import QVariant

# 创建一个新的矢量图层，类型为 MultiLineString
layer = QgsVectorLayer("LineString?crs=EPSG:4326", "MyMultiLineStringLayer", "memory")

# 添加字段
fields = QgsFields()
fields.append(QgsField("id", QVariant.Int))
layer.dataProvider().addAttributes(fields)
layer.updateFields()

# 定义坐标数据
coordinates_1 = [
    (107.498101, 27.123523), (107.498101, 27.123695),
    (107.498101, 27.123695), (107.498101, 27.123875),
    (107.498444, 27.123875), (107.498444, 27.124038),
    (107.498444, 27.124038), (107.498444, 27.12421),
    (107.498444, 27.12421), (107.498444, 27.124381),
    (107.498444, 27.124381), (107.498444, 27.124553),
    (107.498444, 27.124553), (107.498444, 27.124734),
    (107.498787, 27.124734), (107.498787, 27.124896),
    (107.498787, 27.124896), (107.498787, 27.125068),
    (107.498787, 27.125068), (107.498787, 27.125249),
    (107.499131, 27.125249), (107.499131, 27.125411),
    (107.499131, 27.125411), (107.499131, 27.125583),
    (107.499131, 27.125583), (107.499131, 27.125755)
]

coordinates_2 = [
    (107.497414, 27.116828), (107.497414, 27.117),
    (107.497414, 27.117), (107.497414, 27.117172),
    (107.497414, 27.117172), (107.497414, 27.117343),
    (107.497414, 27.117343), (107.497414, 27.117524),
    (107.497757, 27.117524), (107.497757, 27.117687),
    (107.497757, 27.117687), (107.497757, 27.117867),
    (107.498101, 27.117867), (107.498101, 27.11803),
    (107.498101, 27.11803), (107.498101, 27.118202),
    (107.498101, 27.118202), (107.498101, 27.118373),
    (107.498101, 27.118373), (107.498101, 27.118545),
    (107.498101, 27.118545), (107.498101, 27.118725),
    (107.498444, 27.118725), (107.498444, 27.118888),
    (107.498444, 27.118888), (107.498444, 27.11906),
    (107.498444, 27.11906), (107.498444, 27.11924),
    (107.498787, 27.11924), (107.498787, 27.119403),
    (107.498787, 27.119403), (107.498787, 27.119575),
    (107.498787, 27.119575), (107.498787, 27.119755),
    (107.499131, 27.119755), (107.499131, 27.119918),
    (107.499131, 27.119918), (107.499131, 27.12009)
]

# 将坐标转换为 QgsPointXY 对象并创建 MultiLineString
lines = []
for coords in [coordinates_1, coordinates_2]:
    line = [QgsPointXY(x, y) for x, y in coords]
    lines.append(line)

multi_line_geom = QgsGeometry.fromMultiPolylineXY(lines)

# 创建一个特征并设置几何
feature = QgsFeature()
feature.setGeometry(multi_line_geom)
feature.setAttributes([1])  # 设置 ID

# 将特征添加到图层
layer.dataProvider().addFeatures([feature])
layer.updateExtents()

# 将图层添加到 QGIS 中
QgsProject.instance().addMapLayer(layer)