from qgis._core import QgsCoordinateReferenceSystem, QgsCoordinateTransform, QgsProject, QgsPointXY, QgsGeometry, \
    QgsWkbTypes


class tests():
    def geometry_buffer(self, geometry, buffer_value):
        geom_type = geometry.wkbType()  # 获取几何类型
        # Point实现缓冲
        if geom_type == QgsWkbTypes.Point:

            dict_input = self.transform_wgs84_to_utm(geometry.asPoint())
            transformed_point = dict_input['transformed_point']
            utm_epsg = dict_input['zone_number']

            # 创建缓冲区
            transformed_geometry = QgsGeometry.fromPointXY(transformed_point)
            buffer_polygon_utm = transformed_geometry.buffer(buffer_value, 32)

            # 返回缓冲后的图形
            transformed_rings = []
            for ring in buffer_polygon_utm.asPolygon():
                transformed_ring = [self.transform_umt_to_wgs84(pt, utm_epsg) for pt in ring]
                transformed_rings.append(transformed_ring)

            # 创建新特征
            buffer_geometry = QgsGeometry.fromPolygonXY(transformed_rings)
            return buffer_geometry
        # MultiPoint实现缓冲
        elif geom_type == QgsWkbTypes.MultiPoint:
            transformed_points = []
            utm_epsg = None
            for pt in geometry.asMultiPoint():
                dict_input = self.transform_wgs84_to_utm(pt)
                transformed_point = dict_input['transformed_point']
                utm_epsg = dict_input['zone_number']
                transformed_points.append(transformed_point)
            # 形成图形
            transformed_geometry = QgsGeometry.fromMultiPointXY(transformed_points)
            # 创建缓冲区
            buffer_polygon_utm = transformed_geometry.buffer(buffer_value, 32)
            # 返回缓冲后的图形
            transformed_rings = []
            for ring in buffer_polygon_utm.asPolygon():
                transformed_ring = [self.transform_umt_to_wgs84(pt, utm_epsg) for pt in ring]
                transformed_rings.append(transformed_ring)
            # 创建新特征
            buffer_geometry = QgsGeometry.fromPolygonXY(transformed_rings)
            return buffer_geometry
        # Polygon实现缓冲
        elif geom_type == QgsWkbTypes.Polygon:

            transformed_points = []
            utm_epsg = None
            for poly in geometry.asPolygon():
                for pt in poly:
                    dict_input = self.transform_wgs84_to_utm(pt)
                    transformed_point = dict_input['transformed_point']
                    utm_epsg = dict_input['zone_number']
                    transformed_points.append(transformed_point)

            # 形成图形
            transformed_geometry = QgsGeometry.fromPolygonXY([transformed_points])
            # 创建缓冲区
            buffer_polygon_utm = transformed_geometry.buffer(buffer_value, 32)
            buffer_geometry = None
            # 返回缓冲后转会WGS84
            transformed_points = []
            for ring in buffer_polygon_utm.asPolygon():
                transformed_ring = [self.transform_umt_to_wgs84(pt, utm_epsg) for pt in ring]
                transformed_points.append(transformed_ring)
            buffer_geometry = QgsGeometry.fromPolygonXY(transformed_points)

            return buffer_geometry

        elif geom_type == QgsWkbTypes.MultiPolygon:

            transformed_pointslist = []
            utm_epsg = None
            for ring in geometry.asMultiPolygon():
                for poly in ring:
                    transformed_points = []
                    for pt in poly:
                        dict_input = self.transform_wgs84_to_utm(pt)
                        transformed_point = dict_input['transformed_point']
                        utm_epsg = dict_input['zone_number']

                        transformed_points.append(transformed_point)
                    transformed_pointslist.append(transformed_points)

            # 形成图形
            transformed_geometry = QgsGeometry.fromMultiPolygonXY([transformed_pointslist])
            # 创建缓冲区
            buffer_polygon_utm = transformed_geometry.buffer(buffer_value, 32)
            transformed_points = []
            for ring in buffer_polygon_utm.asPolygon():
                transformed_ring = [self.transform_umt_to_wgs84(pt, utm_epsg) for pt in ring]
                transformed_points.append(transformed_ring)
            buffer_geometry = QgsGeometry.fromPolygonXY(transformed_points)
            return buffer_geometry

        # LineString实现缓冲
        elif geom_type == QgsWkbTypes.LineString:
            transformed_points = []
            utm_epsg = None
            for pt in geometry.asPolyline():
                dict_input = self.transform_wgs84_to_utm(pt)
                transformed_point = dict_input['transformed_point']
                utm_epsg = dict_input['zone_number']
                transformed_points.append(transformed_point)
            # 形成图形
            transformed_geometry = QgsGeometry.fromPolylineXY(transformed_points)
            # 创建缓冲区
            buffer_polygon_utm = transformed_geometry.buffer(buffer_value, 32)
            # 返回缓冲后的图形
            transformed_rings = []
            for ring in buffer_polygon_utm.asPolygon():
                transformed_ring = [self.transform_umt_to_wgs84(pt, utm_epsg) for pt in ring]
                transformed_rings.append(transformed_ring)
            buffer_geometry = QgsGeometry.fromPolygonXY(transformed_rings)
            return buffer_geometry
        # MultiLineString实现缓冲
        elif geom_type == QgsWkbTypes.MultiLineString:
            transformed_points = []
            utm_epsg = None
            for poly in geometry.asMultiPolyline():
                for pt in poly:
                    dict_input = self.transform_wgs84_to_utm(pt)
                    transformed_point = dict_input['transformed_point']
                    utm_epsg = dict_input['zone_number']
                    transformed_points.append(transformed_point)
            # 形成图形
            transformed_geometry = QgsGeometry.fromMultiPolylineXY([transformed_points])
            # 创建缓冲区
            buffer_polygon_utm = transformed_geometry.buffer(buffer_value, 32)

            # 返回缓冲后转会WGS84
            transformed_rings = []
            # 转换
            for ring in buffer_polygon_utm.asPolygon():
                transformed_ring = [self.transform_umt_to_wgs84(pt, utm_epsg) for pt in ring]
                transformed_rings.append(transformed_ring)
            # # 创建新特征
            buffer_geometry = QgsGeometry.fromPolygonXY(transformed_rings)
            return buffer_geometry

    def transform_wgs84_to_utm(self, qgis_point):
        longitude = qgis_point.x()
        latitude = qgis_point.y()

        # 判断所处 UTM 带
        zone_number = self.get_utm_zone(longitude)

        # UTM带是326到327之间
        utm_epsg = f'EPSG:326{zone_number}' if latitude >= 0 else f'EPSG:327{zone_number}'

        # 创建坐标参考系统和转换
        wgs84_crs = QgsCoordinateReferenceSystem("EPSG:4326")
        utm_crs = QgsCoordinateReferenceSystem(utm_epsg)

        transform_to_utm = QgsCoordinateTransform(wgs84_crs, utm_crs, QgsProject.instance())
        # 转换点坐标
        transformed_point = transform_to_utm.transform(qgis_point)

        return {
            "transformed_point": QgsPointXY(transformed_point),
            "zone_number": utm_epsg
        }

    @staticmethod
    def transform_umt_to_wgs84(qgis_point_utm, utm_epsg):
        # 创建坐标参考系统和转换
        utm_crs = QgsCoordinateReferenceSystem(utm_epsg)
        wgs84_crs = QgsCoordinateReferenceSystem("EPSG:4326")

        transform_to_wgs = QgsCoordinateTransform(utm_crs, wgs84_crs, QgsProject.instance())
        # 转换点坐标
        transformed_point = transform_to_wgs.transform(qgis_point_utm)
        return transformed_point

    @staticmethod
    def get_utm_zone(longitude):
        return int((longitude + 180) // 6) + 1